#include "includes.h"
#include "debug.h"
#include "network.h"
#include "file.h"

int debug_level;

void debug_print_hex(uint8_t* buffer, int size, char* prefix, int width) {
    int i;
    int pos = 0;
    if(debug_level) {
        for(i = 0; i < size; i++) {
            if(!pos) {
                printf("DEBUG: %s: $%04x: ", prefix, i);
            }
            printf("$%02x ", buffer[i]);
            pos++;
            if(pos == width) {
                printf("\n");
                pos = 0;
            }
        }
        if(pos) {
            printf("\n");
        }
    }
}

void debug_print_packet_in(packet64* p, char* prefix) {
    if(debug_level) {
        switch(p->type) {
            case PACKET_DATA:
                printf("DEBUG: %s: data'\n", prefix);
                debug_print_hex(p->data, p->size, prefix, 8);
            break;
            case PACKET_REQUEST:
                printf("DEBUG: %s: request $%02x bytes\n", prefix, p->req_size);
            break;
            case PACKET_COMMAND:
                printf("DEBUG: %s: command '$%02x'\n", prefix, p->command);
            break;
        }
        printf("DEBUG: %s: received payload: $%X\n", prefix, p->size);
    }
}

void debug_print_packet_out(packet64* p, char* prefix) {
    if(debug_level) {
        switch(p->type) {
            case PACKET_DATA:
                printf("DEBUG: %s: data\n", prefix);
                debug_print_hex(p->data, p->size, prefix, 8);
                printf("DEBUG: %s: error '$%02x'\n", prefix, p->err);
            break;
            case PACKET_ERROR:
                printf("DEBUG: %s: error '$%02x'\n", prefix, p->err);
            break;
        }
        printf("DEBUG: %s: sent payload: $%X\n", prefix, p->size);
    }
}

void debug_start_measure(file64* f) {
    if(ENABLE_BENCHMARK) {
        ftime(&f->starttime);
    }
    return;
}

void debug_end_measure(file64* f) {
    float time = 0;
    float rate = 0;
    long timediff = 0;
    if(ENABLE_BENCHMARK) {
        ftime (&f->endtime);
        timediff = (f->endtime.time * 1000.0 + f->endtime.millitm) - (f->starttime.time * 1000.0 + f->starttime.millitm);

        time = (float)timediff / 1000.0;
        rate = (float)f->pos / time / 1024.0;
        printf("BENCHMARK: %ld bytes transferred in %.4fs => %.2f kb/s\n", f->pos, time, rate);
    }
    return;
}

