#include "server.h"
#include "dir_funcs.h"
#include "macros.h"
#include "debug_funcs.h"
#include "container.h"
#include "file.h"
#include "file_funcs.h"
#include "filetypes.h"
#include "container_funcs.h"

static void dir_buffer_header(file64* f) {
    container* cnt = f->cnt;
    f->err = ERR_OK;
    uint8_t* dirline;
    int i;
    int size = 16 + strlen(cnt->header);

    int pos = 0;

    dirline = calloc(size, sizeof(uint8_t));

    memset(dirline, 0, size);

    /* set start address */
    cnt->dirpointer = 0x0801;
    dirline[0] = cnt->dirpointer & 0xff;
    dirline[1] = cnt->dirpointer >> 8;

    /* set next basic line pointer */
    cnt->dirpointer += size - 2;
    dirline[2] = cnt->dirpointer & 0xff;
    dirline[3] = cnt->dirpointer >> 8;

    /* set reverse on = asc $12 */
    dirline[6] = 0x12;

    /* place " " */
    dirline[7] = '"';
    memcpy(&dirline[8],  cnt->header, strlen(cnt->header));
    pos = 8 + strlen(cnt->header);
    dirline[pos] = '"'; pos++;
    dirline[pos] = ' '; pos++;

    memcpy(&dirline[pos], cnt->id, strlen(cnt->id));

    /* write to buffer */
    for(i = 0; i < size; i++) file_write(f, dirline[i]);
    debug_print_hex(dirline, size, "DIR", 32);
    free(dirline);
}

static void dir_buffer_direntry(file64* f, int num) {
    uint8_t dirline[32];
    char file[MAX_C64_NAME + 3];
    int i;
    int pos;
    container *cnt = f->cnt;
    dirent64* d = cnt->dirlist[num];

    f->err = ERR_OK;

    memset(dirline, 0, 32);

    /* set next basic line pointer */
    cnt->dirpointer += 32;
    dirline[0] = cnt->dirpointer & 0xff;
    dirline[1] = cnt->dirpointer >> 8;
    /* set size information */
    dirline[2] = d->d_blocks & 0xff;
    dirline[3] = d->d_blocks >> 8;
    /* pad depending on size */
    pos = 4;
    if(d->d_blocks < 10)   { dirline[pos] = ' '; pos++; }
    if(d->d_blocks < 100)  { dirline[pos] = ' '; pos++; }
    if(d->d_blocks < 1000) { dirline[pos] = ' '; pos++; }

    /* add " " to filename */
    sprintf(file,"\"%s\"", d->d_petscii_name);
    /* sprintf all together */
    pos += sprintf((char*)dirline + pos, "%-18s%c%s%c", file, d->d_closed ? ' ':'*', filetypes[d->d_type], d->d_locked ? '<' : ' ');
    /* pad remaining bytes */
    while(pos < 31) { dirline[pos] = ' '; pos++; }

    /* write to buffer */
    for(i = 0; i<32; i++) file_write(f, dirline[i]);
    debug_print_hex(dirline, 32, "DIR", 32);
}

static void dir_buffer_tail(file64* f) {
    uint8_t dirline[32];
    int i;
    container *cnt = f->cnt;

    f->err = ERR_OK;

    memset(dirline, 0, 32);

    /* set next basic line pointer */
    cnt->dirpointer += 30;
    dirline[0] = cnt->dirpointer & 0xff;
    dirline[1] = cnt->dirpointer >> 8;
    /* set size information */
    dirline[2] = cnt->free & 0xff;
    dirline[3] = cnt->free >> 8;

    strcpy((char*)&dirline[4], "BLOCKS FREE.             ");

    //write buffer
    for(i = 0; i < 32; i++) file_write(f, dirline[i]);
    debug_print_hex(dirline, 32, "DIR", 32);
}

int dir_open(file64* f) {
    int i;
    container* cnt = f->cnt;
    f->err = ERR_OK;
    if(f->f_mode == OPEN_WRITE) {
        f->err = ERR_FILE_TYPE_MISMATCH;
        return f->err;
    }
    /* handle real files only, no buffered ones (they are always open for read/write) */
    f->err = file_search(f, 0);

    /* enable buffer access */
    f->buffered = 1;

    /* fill buffer with dir */
    dir_buffer_header(f);
    for(i = 0; i < cnt->dirsize; i++) dir_buffer_direntry(f, i);
    dir_buffer_tail(f);
    f->open = 1;
    f->pos = 0;
    debug_message("successful.\n");
    return f->err;
}
