/* dos.c
 * this file handles all dos commands and operations (open, close, set status) just like what you would expect from a for e.g. 1541 dos *
 * dos functions that write on a file, create a own new file, the already present file form channel is used as source file on dos operations */


//array von commands:
//cd, (*dos_change_dir) als functionpointer
//-> mit strcmp(arg1[i]) commands checken und bei erfolg auf pointer gehen, damir funktionsliste leichter erweiterbar
//
//
//
//fe89	Table of command words			DATA
//;	"VIDMBUP&CRSN"
//;	Validate, Initialise, D(Backup unused),
//;	Memory, Block, User,
//;	Position, &and;USR execute, Copy,
//;	Rename, Scratch, New (Format).
//
//ffea	USER vectors
//ffea	UA, U1	[CD5F]				WORD
//ffec	UB, U2	[CD97]				WORD
//ffee	UC, U3	[0500]				WORD
//fff0	UD, U4	[0503]				WORD
//fff2	UE, U5	[0506]				WORD
//fff4	UF, U6	[0509]				WORD
//fff6	UG, U7	[050C]				WORD
//fff8	UH, U8	[050F]				WORD
//fffa	UI, U9	[FF01]				WORD
//
//

#include "dos_funcs.h"
#include "file_funcs.h"
#include "parse_funcs.h"
#include "path_funcs.h"
#include "debug_funcs.h"
#include "container.h"
#include "iec.h"
#include "filetypes.h"
#include "buffer_funcs.h"
#include "string_funcs.h"
#include "container_funcs.h"
#include "dir_funcs.h"
#include "fs_d64_funcs.h"

void dos_set_status(drive* c, int err, int err_track, int err_sector) {
    int i;
    char* status = calloc(1, MAX_FILESPEC + 1);
    sprintf(status,"%02d, %s,%02d,%02d\r", err, error_codes[err], err_track, err_sector);
    buffer_init(c->status);
    for(i = 0; i < strlen(status); i++) {
        buffer_write(c->status, status[i], 32);
    }
    info_message("dos status: %s\n",status);
    free(status);
}

void dos_init(drive* c) {
    dos_set_status(c, ERR_VERSION, 0, 0);
}

int dos_close(drive* c, file64* f) {
    dos_set_status(c, f->err, f->err_track, f->err_sector);
    container_close(f);
    file_close(f);
    return f->err;
}

int dos_rename(drive* c, file64* f) {
//    return file_rename(f, newname);

//    file_rename muss chekcne ob target existiert?
//    und checken ob source existiert
//    dann filesys_specific rename
//
//    file64* tgt;
//    tgt = file_create();
    /*XXX File_init mit file_create verschmelzen? wird ansich nicht mehr gebraucht und wenn nur nach dem create? */
//    file_init(tgt);

    //copy path, f_target nach f_filespec
    //f_mode = OPEN_READ

//source -> file_search -> file_not_found?
//target -> file_search -> file_exists?
//
//ggf funktionen implementieren für: file_exists() und file_available() ?
    return ERR_OK;
}

int dos_copy(drive* c, file64* f) {
    f->err = ERR_OK;
    return f->err;
}

int dos_change_dir(drive* c, file64* f) {
    /* change path on given partition */
    f->err = path_change(c->chroot[f->f_partition], &c->path[f->f_partition], f->f_filespec);
    return f->err;
}

int dos_format_dxx(drive* c, file64* f) {
    if((f->err = container_open(f, c->path[f->f_partition]))) return f->err;
    f->err = fs_d64_format(f->cnt, f->f_header, f->f_id);
    if((f->err = container_close(f))) return f->err;
    return f->err;
}

int dos_create_dxx(drive* c, file64* f) {
    int i;

    debug_message("creating new diskimage @'%s (%s)' filename='%s' header='%s\' id='%s'\n", f->path, container_get_name(f->media), f->f_glob, f->f_header, f->f_id);
    /* are we within fs_sys? */
    if(f->media != CTYPE_SYS) return ERR_CONTAINER_TYPE_MISMATCH;

    f->f_mode = OPEN_WRITE;
    f->filetype = FILETYPE_PRG;
    f->f_overwrite = 0;

    /* try to open */
    if((f->err = file_open(f))) return f->err;

    /* punp 174848 bytes into file */
    for(i = 0; i < 174848; i++) {
        if((f->err = file_write(f, 0))) return f->err;
    }
    if((f->err = file_close(f))) return f->err;

    /* change path */
    if((f->err = path_search(&f->path, c->chroot[f->f_partition], c->path[f->f_partition], f->f_glob))) return f->err;

    /* reopen container again, as path has changed (to our freshly created .d64) */
    if((f->err = container_close(f))) return f->err;
    if((f->err = container_open(f, c->path[f->f_partition]))) return f->err;

    /* now format */
    if((f->err = fs_d64_format(f->cnt, f->f_header, f->f_id))) return f->err;

    return f->err;
}

int dos_scratch(drive* c, file64* f) {
    container* cnt = f->cnt;
    info_message("scratching file(s) '%s'\n", f->f_glob);
    f->err_track = 0;
    /* find all matching files */
    f->err = file_search(f, 0);
    /* scratch all files found */
    for(f->cnt->dirpos = 0; cnt->dirpos < cnt->dirsize; cnt->dirpos++) {
        if(cnt->dirlist[cnt->dirpos]->d_type != FILETYPE_DIR) {
            /* copy current dirent to file */
            file_setup_from_dirent(f, cnt->dirlist[cnt->dirpos]);
            /* try to scratch */
            if((f->err = file_delete(f))) return f->err;
            /* increase number of scratched files */
            f->err_track++;
        }
    }
    f->err = ERR_FILES_SCRATCHED;
    return f->err;
}

/* do the dos specific part of open (like opening a dir, change a dir, file-stuff will be handled in file_open() */
int dos_open(drive* c, file64* f, int mode) {
    if(!f) {
        debug_message("File not allocated. This should not happen!\n");
    }

    /* preset mode and partition */
    if(mode <= 1) f->f_mode = mode;
    else f->f_mode = OPEN_READ;
    f->f_partition = c->curr_partition;

    /* set permissions */
    f->umask = c->umask;
    f->gid = c->gid;
    f->uid = c->uid;

    /* now populate f->f_*-values */
    parse_filespec(f, (char*)c->filespec->data, c->filespec->end);

    /* change path temporarily (respect path component of filespec) */
    f->err = path_search(&f->path, c->chroot[f->f_partition], c->path[f->f_partition], f->f_path);

    if(!f->err) {
        /* as we have the final path now, open corresponding container */
        //XXX TODO open with write/read mode
        f->err = container_open(f, c->path[f->f_partition]);

        if(!f->err) {
            /* open on error channel */
            if(c->lfn == IEC_ERROR_CHANNEL) {
                /* always set source file to read mode during dos_commands */
                f->f_mode = OPEN_READ;
                f->err = dos_command(c, f);
            }
            /* open on file, buffer, dir */
            else {
                if(!f->open) {
                    /* shall we open a dir? */
                    if(f->f_dircmd) {
                        f->err = dir_open(f);
                    }
                    /* or a file? */
                    else {
                        /* try to open file with parsed data */
                        f->err = file_open(f);
                        /* open failed */
                        if(f->err == ERR_FILE_NOT_FOUND || f->err == ERR_FILE_TYPE_MISMATCH || f->err == ERR_FILENAME_MISSING) {
                            /* maybe filename was a path, try to change */
                            f->err = dos_change_dir(c, f);
                            /* success */
                            if(!f->err) {
                                /* serve a dummy file */
                                f->buffered = 1;
                                f->err = buffer_write(f->buffer, 0, 32);
                                f->open = 1;
                            }
                            else {
                                f->err = ERR_FILE_NOT_FOUND;
                            }
                        }
                    }
                }
            }
        }
    }
    else {
        f->err = container_open(f, c->path[f->f_partition]);
        if(!f->err) {
            /* try to open file with full filespec */
            dyn_strcpy(&f->f_glob, f->f_filespec);
            dyn_strcpy(&f->f_path, "");
            f->err = file_open(f);
        }
    }
    /* set status */
    dos_set_status(c, f->err, f->err_track, f->err_sector);
    return f->err;
}

int dos_command(drive* c, file64* f) {
    debug_message("executing '%s'\n", f->f_filespec);

    if(!strcmp(f->f_command, "S") || !strcmp(f->f_command, "SCRATCH") || !strcmp(f->f_command, "RM") ||  !strcmp(f->f_command, "DEL") ||  !strcmp(f->f_command, "DELETE")) {
        f->err = dos_scratch(c, f);
    }
    else if(!strcmp(f->f_command, "CD")) {
        f->err = dos_change_dir(c, f);
    }
    else if(!strcmp(f->f_command, "R") || !strcmp(f->f_command, "RENAME")) {
        f->err = dos_rename(c, f);
    }
    else if(!strcmp(f->f_command, "C") || !strcmp(f->f_command, "COPY")) {
        f->err = dos_copy(c, f);
    }
    else if(!strcmp(f->f_command, "N") || !strcmp(f->f_command, "FORMAT")) {
        f->err = dos_format_dxx(c, f);
    }
    else if(!strcmp(f->f_command, "CREATE")) {
        f->err = dos_create_dxx(c, f);
    }
    else if(!strcmp(f->f_command, "MD") || !strcmp(f->f_command, "MKDIR")) {
        //nur in fs_sys mkdir
        //nur in fs_sys rmdir (nur falls leer)
    }

    else {
        /* unknown command */
        f->err = ERR_SYNTAX_ERROR_30;
    }
    dos_set_status(c, f->err, f->err_track, f->err_sector);
    return f->err;
}

//    //i -> reset drive -> idle_state usw.
//    //mv, mkd64, mkdir? mkdir:gaga.d64,lala,xx? delete, del, r, i? validate? n, format?
//    //valiadate: jedne direntry abklappern und sectorchains als belegt markieren, ggf nopch offenen files schliessen
//    //verbleibende sectors == frei
