#include "drive.h"
#include "drive_funcs.h"
#include "iec_funcs.h"
#include "file_funcs.h"
#include "container_funcs.h"
#include "iec.h"
#include "debug_funcs.h"
#include "network.h"
#include "path_funcs.h"
#include "dos_funcs.h"
#include "string_funcs.h"
#include "buffer_funcs.h"

void drive_destruct(drive* c) {
    int j;
    if(c->name) free(c->name);
    for(j = 0; j < MAX_LOGICAL_FILES; j++) {
        if(c->files[j]) {
            /* close possibly open files and containers */
            file_close(c->files[j]);
            file_destruct(c->files[j]);
            c->files[j] = 0;
        }
    }
    for(j = 0; j < MAX_PARTITIONS; j++) {
        if(c->chroot) free(c->chroot[j]);
        if(c->path) free(c->path[j]);
    }
    buffer_destruct(c->filespec);
    buffer_destruct(c->status);
    c->filespec = NULL;
    c->status = NULL;
    free(c);
}

drive* drive_create(server *s) {
    drive* c = NULL;
    int j;
    //XXX check if drive exists already (comapre names of all drives) if so, do not create new objects
    if(s->num_drives < MAX_DRIVES) {
        c = (drive*)calloc(1, sizeof(drive));
        for(j = 0; j < MAX_LOGICAL_FILES; j++) {
            c->files[j] = NULL;
        }
        for(j = 0; j < MAX_PARTITIONS; j++) {
            c->chroot[j] = NULL;;
            c->path[j] = NULL;;
        }
        c->name = NULL;
        s->drives[s->num_drives] = c;
        s->num_drives++;
    }
    c->filespec = buffer_create();
    c->status = buffer_create();
    return c;
}

void drive_init(drive* c) {
    int j;
    uint8_t ctype;

    info_message("initializing drive '%s' -> %s\n",c->name, c->ip);
    //info_message("  uid: %d  gid: %d  umask: %04o\n",c->uid,c->gid,c->umask);
    //info_message("  ip: %s\n",c->ip);
    for(j = 0; j < MAX_PARTITIONS; j++) {
        if(c->chroot[j] && strlen(c->chroot[j])) {
            //XXX TODO also, we should check if chroot + path is a dir and not a file?
            //pfad als / setzen change_path auf chroot machen, falls okay dann dir + ctype passt!?
            /* find out container type */
            container_get_type(c->chroot[j], &ctype);
            /* reset path */
            path_change(c->chroot[j], &c->path[j], "/");
            //dyn_strcpy(&c->path[j], "");

            info_message("partition [%d]: %s  (container: %s)\n", j, c->chroot[j], container_get_name(ctype));
        }
        else {
            info_message("partition [%d]: (unset)\n", j);
        }
    }
    c->job = IEC_IDLE;
    c->SA = 0;
    c->lfn = 0;
    c->in.type = PACKET_NONE;
    c->out.type = PACKET_NONE;
    dos_init(c);
}

void drive_serve(drive* c) {
    iec_process_packet(c);
    return;
}
