#ifndef ERROR_H
#define ERROR_H

#define ERR_OK                         0
#define ERR_FILES_SCRATCHED            1

#define ERR_FILENAME_MISSING           10

#define ERR_READ_ERROR_20              20
#define ERR_READ_ERROR_21              21
#define ERR_READ_ERROR_22              22
#define ERR_READ_ERROR_23              23
#define ERR_READ_ERROR_24              24
#define ERR_READ_ERROR_27              27
#define ERR_WRITE_ERROR                25
#define ERR_WRITE_PROTECT_ON           26
#define ERR_SYNTAX_ERROR_30            30
#define ERR_SYNTAX_ERROR_31            31
#define ERR_SYNTAX_ERROR_32            32
#define ERR_SYNTAX_ERROR_33            33
#define ERR_SYNTAX_ERROR_34            34
#define ERR_DISK_ID_MISMATCH           29
#define ERR_UNKNOWN_CTYPE              40
#define ERR_CONTAINER_TYPE_MISMATCH    41
#define ERR_WRITE_FILE_OPEN            60
#define ERR_FILE_NOT_OPEN              61
#define ERR_FILE_NOT_FOUND             62
#define ERR_FILE_EXISTS                63
#define ERR_FILE_TYPE_MISMATCH         64
#define ERR_NO_BLOCK                   65
#define ERR_ILLEGAL_TRACK_OR_SECTOR_66 66
#define ERR_ILLEGAL_TRACK_OR_SECTOR_67 67
#define ERR_DIR_NOT_FOUND              68
#define ERR_PARTITION_NOT_FOUND        69
#define ERR_DIR_ERROR                  71
#define ERR_DISK_FULL                  72
#define ERR_VERSION                    73
#define ERR_DRIVE_NOT_READY            74
#define ERR_EOF                        255

static const char error_codes[][32]={
    {"OK"},                      /* 00 */
    {"FILES SCRATCHED"},         /* 01 */
    {""},                        /* 02 */
    {""},                        /* 03 */
    {""},                        /* 04 */
    {""},                        /* 05 */
    {""},                        /* 06 */
    {""},                        /* 07 */
    {""},                        /* 08 */
    {""},                        /* 09 */
    {"FILENAME MISSING"},        /* 10 */
    {""},                        /* 11 */
    {""},                        /* 12 */
    {""},                        /* 13 */
    {""},                        /* 14 */
    {""},                        /* 15 */
    {""},                        /* 16 */
    {""},                        /* 17 */
    {""},                        /* 18 */
    {""},                        /* 19 */
    {"READ ERROR"},              /* 20 */
    {"READ ERROR"},              /* 21 */
    {"READ ERROR"},              /* 22 */
    {"READ ERROR"},              /* 23 */
    {"READ ERROR"},              /* 24 */
    {"WRITE ERROR"},             /* 25 */
    {"WRITE PROTECT ON"},        /* 26 */
    {"READ ERROR"},              /* 27 */
    {"WRITE_ERROR"},             /* 28 */
    {"DISK ID MISMATCH"},        /* 29 */
    {"SYNTAX ERROR"},            /* 30 */
    {"SYNTAX ERROR"},            /* 31 */
    {"SYNTAX ERROR"},            /* 32 */
    {"SYNTAX ERROR"},            /* 33 */
    {"SYNTAX ERROR"},            /* 34 */
    {""},                        /* 35 */
    {""},                        /* 36 */
    {""},                        /* 37 */
    {""},                        /* 38 */
    {"FILE NOT FOUND"},          /* 39 */
    {"UNKNOWN CONTAINER TYPE"},  /* 40 */
    {"CONTAINER TYPE MISMATCH"}, /* 41 */
    {""},                        /* 42 */
    {""},                        /* 43 */
    {""},                        /* 44 */
    {""},                        /* 45 */
    {""},                        /* 46 */
    {""},                        /* 47 */
    {""},                        /* 48 */
    {""},                        /* 49 */
    {"RECORD NOT PRESENT"},      /* 50 */
    {"OVERFLOW IN RECORD"},      /* 51 */
    {"FILE TOO LARGE"},          /* 52 */
    {""},                        /* 53 */
    {""},                        /* 54 */
    {""},                        /* 55 */
    {""},                        /* 56 */
    {""},                        /* 57 */
    {""},                        /* 58 */
    {""},                        /* 59 */
    {"WRITE FILE OPEN"},         /* 60 */
    {"FILE NOT OPEN"},           /* 61 */
    {"FILE NOT FOUND"},          /* 62 */
    {"FILE EXISTS"},             /* 63 */
    {"FILE TYPE MISMATCH"},      /* 64 */
    {"NO BLOCK"},                /* 65 */
    {"ILLEGAL TRACK OR SECTOR"}, /* 66 */
    {"ILLEGAL TRACK OR SECTOR"}, /* 67 */
    {"DIR NOT FOUND"},           /* 68 */    //specific
    {"PARTITION NOT FOUND"},     /* 69 */    //specific
    {"NO CHANNEL"},              /* 70 */
    {"DIR ERROR"},               /* 71 */
    {"DISK FULL"},               /* 72 */
    {"64NET/3 NG V0.1"},         /* 73 */
    {"DRIVE NOT READY"},         /* 74 */
    {""},                        /* 75 */
    {""},                        /* 76 */
    {""},                        /* 77 */
    {""},                        /* 78 */
    {""},                        /* 79 */
    {""},                        /* 80 */
    {""},                        /* 81 */
    {""},                        /* 82 */
    {""},                        /* 83 */
};


//CODE    MEANING                DOS ERROR MESSAGE
//01    Everything OK            00, OK
//02    Header block not found        20, READ ERROR
//03    SYNC not found            21, READ ERROR
//04    Data block not found        22, READ ERROR
//05    Checksum error in data block    23, READ ERROR
//07    Verify error            25, WRITE ERROR
//08    Disk write protected        26, WRITE PROTECT ON
//09    Checksum error in header block    27, READ ERROR
//0B    Id mismatch            29, DISK ID MISMATCH
//0F    Disk not inserted        74, DRIVE NOT READY
//
//;e4fc    00,    Ok
//;e500    20,21,22,23,24,27, Read error
//;e50b    52,    File too large
//;e517    50,    Record not present
//;e522    51,    Overflow in record
//;e52f    25,28,    Write error
//;e533    26,    Write protect on
//;e540    29,    Disk id mismatch
//;e546    30,31,32,33,34, Syntax error
//;e552    60,    Write file open
//;e556    63,    File exists
//;e55f    64,    File type mismatch
//;e567    65,    No block
//;e570    66,67,    Illegal track or sector
//;e589    61,    File not open
//;e58d    39,62,    File not found
//;e592    01,    Files scratched
//;e59f    70,    No channel
//;e5aa    71,    Dir error
//;e5af    72,    Disk full
//;e5b6    73,    Cbm dos v2.6 1541
//;e5c8    74,    Drive not ready
//

typedef struct dos_status {
    int err;
    int num1;
    int num2;
    char* msg;
} dos_status;

#endif /* ERROR_H */
