#ifndef FILE_H
#define FILE_H

#include "includes.h"
#include "sizes.h"
#include "fs_d64.h"
#include "fs_sys.h"
#include "error.h"
#include "container.h"
#include "dirent.h"
#include "buffer.h"

#define OPEN_READ       0
#define OPEN_WRITE      1

typedef struct file64 {
    /* flag if data can be retrieved/written from/to a file or container, or just from/to buffer */
    FILE* file;
    int buffered;
    buffer* buffer;

    char *ufs_name;
    char *petscii_name;
    char *path;

    /* starting track of file */
    uint8_t track;
    /* starting sector of file */
    uint8_t sector;

    /* track of direntry */
    uint8_t detrack;
    /* sector of direntry */
    uint8_t desector;
    /* position within dirsector */
    uint8_t desectpos;

    uint16_t blocks;
    size_t size;
    uint8_t closed;
    uint8_t locked;

    //whole filespec
    char* f_filespec;
    //extracted filename
    char* f_glob;
    //extracted path
    char* f_path;
    //extracted command
    char* f_command;
    //left portion of filename "target=source"
    char* f_target;
    //right portion
    char* f_source;
    //header information
    char* f_header;
    //id
    char* f_id;
    //partition
    int f_partition;
    //fielspec contains a dir command
    int f_dircmd;
    //open mode (read/write)
    int f_mode;
    //can file be overwritten on write?
    int f_overwrite;
    //desired filetype
    int f_filetype;

    int open;

    uint8_t filetype;
    uint8_t media;

    int err;
    int err_track;
    int err_sector;

    /* read/write position within file */
    long int pos;

    /* surrounding container, normal filesys, d64, ... */
    container* cnt;

    /* benchmarking */
    struct timeb starttime;
    struct timeb endtime;

    /* ownership things */
    __uid_t uid;
    __gid_t gid;
    int umask;
} file64;

#endif /* FILE_H */
