//open
//close
//get_filetype (dir|file)
//rename
//delete
//create (dir, file (prg|dXX))
//read_byte
//get_byte

#include "container.h"
#include "file.h"
#include "parse_funcs.h"
#include "macros.h"
#include "debug_funcs.h"
#include "filetypes.h"
#include "sizes.h"
#include "error.h"
#include "file_funcs.h"
#include "dir_funcs.h"
#include "petscii_funcs.h"
#include "container_funcs.h"
#include "path_funcs.h"
#include "fs_sys.h"
#include "string_funcs.h"
#include "dirent_funcs.h"

//static int fs_sys_compare(const dirent64** a, const dirent64** b) {
//    return (strcmp((*a)->d_ufs_name, (*b)->d_ufs_name));
//}

int fs_sys_close(file64* f) {
    //close file if still open
    if(f->file != NULL) {
        fclose(f->file);
        f->file = NULL;
    }
    f->err = ERR_OK;
    return f->err;
}

int fs_sys_get_header(container* cnt, char* path) {
    /* add slash for better understanding */
    dyn_strcpy(&cnt->header, "/");
    dyn_strcat(&cnt->header, path);
    ascii2petscii(cnt->header);
    strcpy(cnt->id, "64NET");
    debug_message("header information extracted header='%s' id='%s'\n", cnt->header, cnt->id);
    return ERR_OK;
}

static int fs_sys_closedir(file64* f) {
    if(closedir(f->cnt->dir)) {
        f->err = ERR_DRIVE_NOT_READY;
    }
    return f->err;
}

//unfortunatedly we have to read in the full dir first, so we are able to sort it!
int fs_sys_findfirst(file64* f, char* f_glob) {
    struct dirent* d;
    int i=0;
    struct stat buf;
    char* path = NULL;
    char* glob = NULL;
    container* cnt = f->cnt;
    int err;

    //no direntries ready, so read in dir first;
    if((cnt->dir = opendir(f->path)) == NULL) {
        switch(errno) {
            case EACCES:
            break;
            case EMFILE:
            break;
            case ENFILE:
            break;
            case ENOMEM:
            break;
            case ENOENT:
            break;
        }
        f->err = ERR_DIR_NOT_FOUND;
        return f->err;
    }

    /* work on a copy of glob */
    dyn_strcpy(&glob, f_glob);
    petscii2ascii(glob);

    err = ERR_OK;

    debug_message("opened dir '%s', searchmask is '%s'\n", f->path, glob);
    while((d = readdir(cnt->dir)) != NULL) {
        if(file_select(d->d_name, glob)) {
            cnt->dirlist = (struct dirent64**)realloc((void*)(cnt->dirlist), (size_t)((i + 1)*sizeof(struct dirent64*)));
            if(cnt->dirlist == NULL) {
                err = ERR_DRIVE_NOT_READY;
                break;
            }
            (cnt->dirlist)[i] = dirent_create();
            if((cnt->dirlist)[i] == NULL) {
                err = ERR_DRIVE_NOT_READY;
                break;
            }
            //create c64 name
            strncpy(cnt->dirlist[i]->d_petscii_name, d->d_name, MAX_C64_NAME); cnt->dirlist[i]->d_petscii_name[MAX_C64_NAME] = '\0';
            ascii2petscii(cnt->dirlist[i]->d_petscii_name);
            //copy realname
            strncpy((cnt->dirlist)[i]->d_ufs_name, d->d_name, MAX_FILESPEC);

            /* get container type */
            cnt->dirlist[i]->d_ctype = CTYPE_SYS;
            if((err = container_get_type(d->d_name, &cnt->dirlist[i]->d_ctype))) {
                break;
            }
            if(d->d_type != DT_DIR) {
                if(cnt->dirlist[i]->d_ctype != CTYPE_SYS) {
                    cnt->dirlist[i]->d_type  = FILETYPE_DIR;
                } else {
                    cnt->dirlist[i]->d_type  = FILETYPE_PRG;
                }
            }
            else {
                cnt->dirlist[i]->d_type = FILETYPE_DIR;
            }

            path_assemble(&path, f->path, d->d_name);

            stat(path, &buf);
            cnt->dirlist[i]->d_size = buf.st_size;
            cnt->dirlist[i]->d_closed = 1;
            if(buf.st_size / (SECTOR_SIZE - 2) + 1 > 65535) cnt->dirlist[i]->d_blocks = 65535;
            else cnt->dirlist[i]->d_blocks = buf.st_size / (SECTOR_SIZE - 2) + 1;
            cnt->dirlist[i]->d_locked = (buf.st_mode & S_IWRITE) == 0;
            debug_message("found direntry: '%s' size='%d bytes (%d blocks)' locked='%d' closed='%d' filetype='%s' ufs_name='%s' cnt_type='%s'\n",
                          cnt->dirlist[i]->d_petscii_name, cnt->dirlist[i]->d_size, cnt->dirlist[i]->d_blocks, cnt->dirlist[i]->d_locked, cnt->dirlist[i]->d_closed, filetypes[cnt->dirlist[i]->d_type], cnt->dirlist[i]->d_ufs_name, container_get_name(cnt->dirlist[i]->d_ctype));
            i++;
        }
    }
    /* close dir and save error if no previous error occurred */
    fs_sys_closedir(f);
    if(!err) err = f->err;

    if(path) free(path);
    if(glob) free(glob);

    cnt->dirsize = i;
    debug_message("found %d entries'\n", cnt->dirsize);
    //qsort((void*)(cnt->dirlist), (size_t)i, sizeof(struct dirent64*), (void*)fs_sys_compare);
    f->err = err;
    return f->err;
}

int fs_sys_get_free(container *cnt) {
    cnt->free = 65535;
    return ERR_OK;
}

int fs_sys_rename(file64* f, char* newname) {
    return ERR_OK;
}

int fs_sys_delete(file64* f) {
    char* path = NULL;
    f->err = ERR_OK;

    path_assemble(&path, f->path, f->ufs_name);

    if(unlink(path)) {
        f->err = ERR_WRITE_PROTECT_ON;
    }
    if(path) free(path);

    return f->err;
}

int fs_sys_fseek(file64* f, long int newpos) {
    f->err = ERR_OK;
    fseek(f->file, newpos, SEEK_SET);
    f->pos = newpos;
    return f->err;
}

int fs_sys_create(file64* f) {
    char* path = NULL;
    int res;
    f->err = ERR_OK;

    path_assemble(&path, f->path, f->ufs_name);

    //overwrites automagically
    if((f->file = fopen(path,"wb")) == NULL) {
        debug_message("create failed. errno='%s'  path='%s'\n", strerror(errno), path);
        f->err = ERR_WRITE_PROTECT_ON;
    }
    debug_message("file permissions: %d.%d %o\n", f->uid, f->gid, f->umask);
    //XXX TODO errorhandling
    res = chown(path, f->uid, f->gid);
    res = chmod(path, f->umask);

    if(path) free(path);

    return f->err;
}

int fs_sys_open(file64* f) {
    char* path = NULL;
    f->err = ERR_OK;

    path_assemble(&path, f->path, f->ufs_name);

    if((f->file=fopen(path,"rb"))==NULL) {
        f->err = ERR_FILE_NOT_FOUND;
    }
    if(path) free(path);

    return f->err;
}

int fs_sys_read_block(file64* f, char* data, long int* num_bytes) {
    return ERR_OK;
}

int fs_sys_write_block(file64* f, char* data, long int* num_bytes) {
    return ERR_OK;
}

int fs_sys_eof(file64* f) {
    f->err = ERR_OK;
    long int pos = ftell(f->file);
    if(pos >= f->size) f->err = ERR_EOF;
    return f->err;
}

//depending on type either read from file or do a scandir and read from filelist
int fs_sys_read(file64* f, uint8_t* data) {
    f->err = ERR_OK;
    if(f->f_dircmd) {
        f->err = ERR_READ_ERROR_20;
    } else {
        *data = (uint8_t)fgetc(f->file);
        /* unhappily we have to check for eof on each read, as eof must be set already on last byte being read, the c64 requests it like that */
        f->err = fs_sys_eof(f);
    }
    return f->err;
}

int fs_sys_write(file64* f, uint8_t data) {
    f->err = ERR_OK;
    if(f->f_dircmd) {
        f->err = ERR_WRITE_ERROR;
    } else {
        fputc(data, f->file);
    }
    return f->err;
}
