#include "network.h"
#include "debug_funcs.h"
#include "network_funcs.h"

void network_assemble_packet(packet64* p, uint8_t* buf, int* buf_size) {
    *buf_size = 0;
    buf[*buf_size] = p->type;
    (*buf_size)++;
    switch(p->type) {
        case PACKET_DATA:
            memcpy(&buf[*buf_size], p->data, p->size);
            *buf_size += p->size;
        case PACKET_ERROR:
            buf[*buf_size] = p->err;
            (*buf_size)++;
        break;
    }
}

void network_send_packet(drive* c, uint8_t* buf, int buf_size) {
    sendto(c->sender_fd, buf, buf_size, 0, (struct sockaddr *) &c->sender, sizeof(struct sockaddr));
}

void network_receive_packet(server *s) {
    int flags = MSG_WAITALL;
    s->from_len = 16;
    debug_message("listening on socket...%d\n", ARRAY_ELEMS(s->buffer));
    s->buf_size = recvfrom(s->receiver_fd, s->buffer, MAX_MTU + 4, flags, &s->from, &s->from_len);
    debug_message("received something...\n");
}

void network_analyze_packet(packet64* p, uint8_t* buf, int buf_size) {
    /* set packet type */
    p->type = buf[0];
    /* assemble packet, depending on type */
    switch(p->type) {
        case PACKET_DATA:
            p->size = buf_size - 1;
            if(p->size > MAX_MTU) p->size = MAX_MTU;
            memcpy(p->data, buf + 1, p->size);
        break;
        case PACKET_REQUEST:
            p->size = 0;
            p->req_size = buf[1];
            p->req_size += buf[2] << 8;
            if(p->req_size > MAX_MTU) p->req_size = MAX_MTU;
        break;
        case PACKET_COMMAND:
            p->size = 0;
            p->command = buf[1];
        break;
    }
}
