//cd0:gaga
//cd:0:/gaga
//cd:/gaga
//cd/gaga

#include "parse_funcs.h"
#include "debug_funcs.h"
#include "string_funcs.h"
#include "path_funcs.h"
#include "filetypes.h"
#include "buffer.h"

//ggf aufbohren mit globlen/strlen(glob)?!
int file_select(const char* d_name, const char* glob) {
    int res;
    //no filename == no match, sorry
    if(!glob || !*glob) return 0;
    res = fnmatch(glob, d_name, 0);
    if(res == 0) return 1;
    return 0;
}

/* parses in fielspec-buffer and fills all f->f_* variables
 * also it sets the corresponding path and chroot in container */
int parse_filespec(file64* f, char* filespec, int filespec_len) {
    char* part1 = NULL;
    char* part2 = NULL;
    char* target = NULL;
    char* source = NULL;
    char* temp = NULL;
    char* rest = NULL;
    char* flag = NULL;
    char command[16 + 1] = { 0 };
    int i;

    //XXX also set media depending on path?!
    //zerlegen in command, partition, path/filename, bzw. neuer filespeci n der der verbleibende pfad bleibt?
    //filespec auch in target und source zerlegen (gaga.prg=/blubb/lala.prg)?
    //no : found, so we have only a filename to handle;
    f->f_dircmd = 0;
    f->f_overwrite = 0;
    f->f_filetype = FILETYPE_PRG;

    dyn_strncpy(&temp, filespec, filespec_len);
    debug_message("filespec='%s'\n", temp);
    if(!strlen(temp)) return ERR_OK;

    part1 = strtok(temp,":");
    part2 = strtok(NULL,":");
    if(!part2) {
        //we have no separator found, all is filename
        if(part1) {
            //analyse leading chars and skip them
            if(part1[0]=='$') {
                f->f_dircmd = 1;
                //if a glob follows the $, remove $
                if(part1[1]) part1++;
                //else set * as glob
                else part1[0] = '*';
            }
            //save whole first part
            dyn_strcpy(&rest, part1);
        }
    }
    else {
        //found a separator ':'
        //save whole second part
        dyn_strcpy(&rest, part2);

        //analyse right part further
        //now let's see if we can split further?
        //target can only be a simple filename
        target = strtok(part2, "=");
        //source can be a complete new full filespec
        source = strtok(NULL, "=");
        //we have a source, so we have two parts

        //now analyse left part (command and so on)
        if(part1) {
            //does it contain partition information?
            sscanf(part1, "%16[^0-9]%d", command, &f->f_partition);
            //and a $ sign?
            if(part1[0] == '$') f->f_dircmd = 1;
            else if(part1[0] == '@') f->f_overwrite = 1;
        }
    }
    dyn_strcpy(&f->f_source, source);
    dyn_strcpy(&f->f_target, target);
    dyn_strcpy(&f->f_command, command);

    //now take remaining filespec
    //XXX TODO also see, if remaining filename has still pathinformation -> in case split off path from filename and set separatedly, so it can be processed by change_path later on
//    dyn_strcpy(&f->f_filespec, rest);
    i = strlen(rest);
    while(i) {
        if(rest[i - 1] == '/') break;
        i--;
    }
    dyn_strcpy(&f->f_glob, rest + i);
    dyn_strncpy(&f->f_path, rest, i);

    /* now also split up filename, in case it is gaga.d64,diskimage,xx */
    dyn_strcpy(&temp, f->f_glob);
    //and seperate by ',' to find out things for creating diskimages
    i = 0;
    flag = strtok(temp, ",");
    while(flag) {
        if(i == 0) dyn_strcpy(&f->f_glob, flag);
        if(i == 1) dyn_strcpy(&f->f_header, flag);
        if(i == 2) dyn_strncpy(&f->f_id, flag, 5);
        /* flags can't be first argument, must be preceeding */
        if(flag != temp) {
            debug_message("flag='%s'\n", flag);
            if(flag[0] == 'R') f->f_mode = OPEN_READ;
            if(flag[0] == 'W') f->f_mode = OPEN_WRITE;
            if(flag[0] == 'P') f->f_filetype = FILETYPE_PRG;
            if(flag[0] == 'S') f->f_filetype = FILETYPE_SEQ;
        }
        /* search next element */
        flag = strtok(NULL, ",");
        i++;
    }

    /* glue path to glob again and set as fielspec (usefull for open on .d64) */
    dyn_strcpy(&f->f_filespec, f->f_path);
    dyn_strcat(&f->f_filespec, f->f_glob);

    debug_message("result: filespec='%s', path='%s', filename='%s', cmd='%s', dir='%d', par='%d', target='%s', source='%s', overwrite='%d', header='%s', id='%s'\n",
                  f->f_filespec, f->f_path, f->f_glob, f->f_command, f->f_dircmd, f->f_partition, f->f_target, f->f_source, f->f_overwrite, f->f_header, f->f_id);

    free(temp);
    free(rest);

    return 1;
}
