#include "server.h"
#include "iec.h"
#include "debug.h"
#include "network_funcs.h"
#include "debug_funcs.h"
#include "drive_funcs.h"

static drive* server_dispatch(server *s, uint8_t* source_ip) {
    int c;
    struct hostent* hp;

    /* my really cheap dispatcher *cough*
     * it would be more wisely to make a extra table that has
     * the last number of the ip as index, as all drives are
     * in the same subnet. But so far this should be enough :-) */

    if(s->inject_mode) return s->drives[0];

    for(c = 0; c < s->num_drives; c++) {
        hp = gethostbyname(s->drives[c]->ip);
        if(memcmp(hp->h_addr, source_ip, 4) == 0) {
            return s->drives[c];
        }
    }
    return NULL;
}

void server_destruct(server *s) {
    int c;
    for(c = 0; c < s->num_drives; c++) {
        drive_destruct(s->drives[c]);
        s->drives[c] = NULL;
    }
    free(s->buffer);
    s->buffer = NULL;
    free(s);
}

server* server_create() {
    server* s = (server*)calloc(1, sizeof(server));
    s->num_drives = 0;
    s->buffer = calloc(1, MAX_MTU + 4);
    return s;
}

int server_init(server* s) {
    int c;
    struct hostent *hp;
    //install receiver socket
    s->receiver.sin_family      = AF_INET;
    s->receiver.sin_port        = htons(s->port);
    s->receiver.sin_addr.s_addr = htonl(INADDR_ANY);
    s->receiver_fd              = socket(AF_INET,SOCK_DGRAM, IPPROTO_UDP);

    //bind input socket
    bind(s->receiver_fd, (struct sockaddr *) &s->receiver, sizeof(struct sockaddr));

    if (s->receiver_fd<0) {
        fail_message("Failed to bind to port %d to listen for UDP packets.\n",s->port);
        return 1;
    }

    //install sender sockets for all drives
    for(c = 0; c < s->num_drives; c++) {
        hp = gethostbyname(s->drives[c]->ip);
        s->drives[c]->sender.sin_family = AF_INET;
        s->drives[c]->sender.sin_port   = htons(s->port);
        s->drives[c]->sender_fd         = socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);
        bcopy(hp->h_addr, &(s->drives[c]->sender.sin_addr.s_addr), hp->h_length);
        if (s->drives[c]->sender_fd < 0) {
            fail_message("Failed to bind socket for drive %s.\n",s->drives[c]->name);
            return 1;
        }
        drive_init(s->drives[c]);
    }
    return 0;
}

static void server_tick(server *s) {
    s->curr_drive = NULL;
    if(!s->inject_mode) network_receive_packet(s);
    debug_message("searching for drive...\n");
    s->curr_drive = server_dispatch(s, (uint8_t*)s->from.sa_data + 2);
    if(s->curr_drive!=NULL) {
        debug_message("drive '%s' handling packet...\n", s->curr_drive->name);
        debug_print_hex(s->buffer, s->buf_size, "NETIN", 8);
        network_analyze_packet(&s->curr_drive->in, s->buffer, s->buf_size);
        debug_print_packet_in(&s->curr_drive->in,"IN");
        debug_message("IN: packet for drive %s  buf_size: %d\n",s->curr_drive->name, s->buf_size);
        drive_serve(s->curr_drive);
        /* anything to send? */
        if(s->curr_drive->out.type != PACKET_NONE) {
            debug_print_packet_out(&s->curr_drive->out,"OUT");
            network_assemble_packet(&s->curr_drive->out, s->curr_drive->buffer, &s->curr_drive->buf_size);
            if(!s->inject_mode) network_send_packet(s->curr_drive, s->curr_drive->buffer, s->curr_drive->buf_size);
            debug_print_hex(s->curr_drive->buffer, s->curr_drive->buf_size, "NETOUT", 8);
            s->curr_drive->out.type = PACKET_NONE;
        }
        else {
            if(s->inject_mode) fail_message("INJECT: No reply received!\n");
        }
    }
}

void server_start(server *s) {
    int i;
    info_message("server version v%s\n",VERSION);
    if(s->inject_mode) {
        info_message("server running in INJECT-mode!\n");


        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x49");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x6f");
        server_tick(s);
        s->buf_size = 3;
        strcpy((char*)s->buffer, "R\x80\x00");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x5f");
        server_tick(s);

        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x49");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x6f");
        server_tick(s);
        s->buf_size = 3;
        strcpy((char*)s->buffer, "R\x80\x00");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x5f");
        server_tick(s);

//        s->buf_size = 2;
//        strcpy((char*)s->buffer, "C\x29");
//        server_tick(s);
//        s->buf_size = 2;
//        strcpy((char*)s->buffer, "C\xef");
//        server_tick(s);
//        s->buf_size = 2;
//        strcpy((char*)s->buffer, "C\x3f");
//        server_tick(s);


/*
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x29");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\xff");
        server_tick(s);
        s->buf_size = 30;
        strcpy((char*)s->buffer, "DCREATE0:DRECKSAU.D64,GNAA,XYZ");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x3f");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x29");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\xef");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x3f");
        server_tick(s);

*/
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x29");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\xf0");
        server_tick(s);
        s->buf_size = 13;
        strcpy((char*)s->buffer, "Dtest.a64");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x3f");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x49");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x60");
        server_tick(s);
        for(i = 0; i < 1; i++) {
            s->buf_size = 3;
            s->buffer[0] = 'R';
            s->buffer[1] = 0x00;
            s->buffer[2] = 0x04;
            server_tick(s);

//            s->buf_size = 3;
//            s->buffer[0] = 'R';
//            s->buffer[1] = 0x00;
//            s->buffer[2] = 0x05;
//            server_tick(s);
        }

        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x5f");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x29");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\xe0");
        server_tick(s);
        s->buf_size = 2;
        strcpy((char*)s->buffer, "C\x3f");
        server_tick(s);

    }
    else {
        while(1) {
            server_tick(s);
        }
    }
    return;
}
